package com.ningdatech.pmapi.projectdeclared.converter;

import cn.hutool.core.collection.CollUtil;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.model.entity.ProjectApplication;
import com.ningdatech.pmapi.provincial.model.dto.ProvincialApplicationDTO;
import com.ningdatech.pmapi.provincial.model.dto.ProvincialProjectDTO;
import org.assertj.core.util.Lists;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Classname ApplicationConverter
 * @Description
 * @Date 2023/3/13 17:38
 * @Author PoffyZhang
 */
public class ApplicationConverter {

    public static ProvincialProjectDTO convertProject(Project projectInfo,List<ProjectApplication> applications) {
        return ProvincialProjectDTO.builder()
                .regionCode(projectInfo.getAreaCode())
                .regionName(projectInfo.getArea())
                .projectName(projectInfo.getProjectName())
                .projectId(String.valueOf(projectInfo.getId()))
                .projectType(projectInfo.getProjectType())
                .totalMoney(projectInfo.getDeclareAmount())
                .yearBudget(projectInfo.getAnnualPlanAmount())
                .budgetFrom(projectInfo.getDeclareHaveAmount() + "," +
                        projectInfo.getDeclareGovOwnFinanceAmount() + ","
                        + projectInfo.getDeclareGovSuperiorFinanceAmount())
                .year(String.valueOf(projectInfo.getProjectYear()))
                .financialCode(projectInfo.getFinancialCode())
                .developCode(projectInfo.getDevelopCode())
                .beginTime(projectInfo.getBeginTime())
                .endTime(projectInfo.getEndTime())
                .buildBasis("立项依据")
                .buildBasisFile(projectInfo.getBuildBasis())
                .projectSummary(projectInfo.getProjectIntroduction())
                .responsibleMan(projectInfo.getResponsibleMan())
                .responsibleManPhone(projectInfo.getResponsibleManMobile())
                .contactName(projectInfo.getContactName())
                .contactPhone(projectInfo.getContactPhone())
                .buildUnit(projectInfo.getBuildOrgName())
                .buildUnitCode(projectInfo.getBuildOrgCode())
                .superUnit(projectInfo.getSuperOrg())
                .superUnitCode(projectInfo.getSuperOrgCode())
                .projectApplyFile(projectInfo.getProjectPdf())
                .projectEstimateFile(projectInfo.getCalculationTotalInvestmentFile())
                .unitThreePlan(projectInfo.getMainResponsibilitiesApplicantFile())
                .otherFile(projectInfo.getPreliminaryPlanFile())
                .projectRemark(projectInfo.getProjectRemarks())
                .includeApplication(projectInfo.getIncludeApplication())
                .applicationInfo(convertApplications(applications))
                .build();
    }

    //放入项目 app
    private static List<ProvincialApplicationDTO> convertApplications(List<ProjectApplication> applications) {
        if(CollUtil.isEmpty(applications)){
            Collections.emptyList();
        }

        return applications.stream().map(ApplicationConverter::convertApp).collect(Collectors.toList());
    }

    private static ProvincialApplicationDTO convertApp(ProjectApplication projectApplication) {
        return ProvincialApplicationDTO.builder()
                .clouds(convertCloud(projectApplication))
                .isFirst(projectApplication.getIsFirst())
                .applicationName(projectApplication.getApplicationName())
                .applicationCode(projectApplication.getRelatedExistsApplicationCode())
                .relatedExistsApplication(projectApplication.getRelatedExistsApplication())
                .applicationType(2)
                .buildLevel(projectApplication.getBuildLevel())
                .isUniteBuild(projectApplication.getIsUniteBuild())
                .unionBuildKind(projectApplication.getUnionBuildKind())
                .applicationSummary(projectApplication.getApplicationSummary())
                .applicationRemark(projectApplication.getApplicationRemark())
                .applicationEstimateFile(projectApplication.getApplicationEstimateFile())
                .isFiveDomain(projectApplication.getIsDigitalModification())
                .fiveDomain(projectApplication.getDigitalModification())
                .bizDomain(projectApplication.getBizDomain())
                .isBizCooperate(projectApplication.getIsBizCooperate())
                .userRange(projectApplication.getUsesRangeRemark())
                .useGovCloud(projectApplication.getUseGovCloud())
                .nationalITSpec(projectApplication.getNationalItSpec())
                .netEnv(String.valueOf(projectApplication.getNetEnv()))
                .secrecyGrade(projectApplication.getSecrecyGrade())
                .passwordGrade(projectApplication.getPasswordGrade())
                .accountAppName(projectApplication.getAccountAppName())
                .brainAccountAppName(projectApplication.getDomainBrainAccount())
                .useCommonData(projectApplication.getUseCommonData())
                .dataName(projectApplication.getDataName())
                .commonComponents(projectApplication.getCommonComponents())
                .useCommonComponent(projectApplication.getUseCommonComponent())
                .isProduceCommonComponent(projectApplication.getProduceCommonComponent())
                .produceCommonComponent(projectApplication.getProduceCommonComponents())
                .publishSide(projectApplication.getPublishSide())
                .build();
    }

    private static List<ProvincialApplicationDTO.Cloud> convertCloud(ProjectApplication projectApplication) {
        return Lists.newArrayList(ProvincialApplicationDTO.Cloud.builder()
                        .cloudType(projectApplication.getCloudsType())
                        .cloudNums(projectApplication.getCloudsNumber())
                        .cloudBasicSpec(projectApplication.getCloudsFoundationSpecifications())
                        .cloudUseDescription(projectApplication.getCloudsDescription())
                        .build());
    }

}
