package com.ningdatech.pmapi.user.manage;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.pmapi.common.constant.RegionConst;
import com.ningdatech.pmapi.common.helper.RegionCacheHelper;
import com.ningdatech.pmapi.common.helper.UserInfoHelper;
import com.ningdatech.pmapi.organization.model.entity.DingEmployeeInfo;
import com.ningdatech.pmapi.organization.model.entity.DingOrganization;
import com.ningdatech.pmapi.organization.service.IDingEmployeeInfoService;
import com.ningdatech.pmapi.organization.service.IDingOrganizationService;
import com.ningdatech.pmapi.sys.model.dto.RegionDTO;
import com.ningdatech.pmapi.sys.model.entity.Role;
import com.ningdatech.pmapi.sys.model.entity.UserRole;
import com.ningdatech.pmapi.sys.service.IRoleService;
import com.ningdatech.pmapi.sys.service.IUserRoleService;
import com.ningdatech.pmapi.user.constant.UserAvailableEnum;
import com.ningdatech.pmapi.user.entity.UserInfo;
import com.ningdatech.pmapi.user.model.po.ReqUserDetailEditPO;
import com.ningdatech.pmapi.user.model.po.ReqUserDetailPO;
import com.ningdatech.pmapi.user.model.po.ReqUserDisableOrEnablePO;
import com.ningdatech.pmapi.user.model.po.ReqUserInfoListPO;
import com.ningdatech.pmapi.user.model.vo.ResUserDetailVO;
import com.ningdatech.pmapi.user.model.vo.ResUserInfoListVO;
import com.ningdatech.pmapi.user.model.vo.UserRoleVO;
import com.ningdatech.pmapi.user.security.auth.model.UserFullInfoDTO;
import com.ningdatech.pmapi.user.service.IUserInfoService;
import com.ningdatech.pmapi.user.util.LoginUserUtil;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author liuxinxin
 * @date 2023/2/13 上午9:09
 */
@Component
@RequiredArgsConstructor
public class UserInfoManage {

    private final IDingOrganizationService iDingOrganizationService;
    private final IDingEmployeeInfoService iDingEmployeeInfoService;
    private final IUserInfoService iUserInfoService;
    private final IUserRoleService iUserRoleService;
    private final IRoleService iRoleService;
    private final UserInfoHelper userInfoHelper;

    private final RegionCacheHelper regionCacheHelper;

    public PageVo<ResUserInfoListVO> list(ReqUserInfoListPO reqUserInfoListPO) {
        PageVo<ResUserInfoListVO> pageVo = new PageVo<>();

        String phoneNo = reqUserInfoListPO.getPhoneNo();
        String orgCode = reqUserInfoListPO.getOrgCode();
        String name = reqUserInfoListPO.getName();

        List<String> roleCompliantEmployeeCodeList = getRoleCompliantEmployeeCodeList(reqUserInfoListPO.getUserRoleList());
        if (roleCompliantEmployeeCodeList != null && roleCompliantEmployeeCodeList.size() == 0) {
            return pageVo;
        }
        List<String> orgNameCompliantOrgCodeList = getOrgNameCompliantOrgCodeList(reqUserInfoListPO.getOrgName());
        if (orgNameCompliantOrgCodeList != null && orgNameCompliantOrgCodeList.size() == 0) {
            return pageVo;
        }

        LambdaQueryWrapper<DingEmployeeInfo> wrapper = Wrappers.lambdaQuery(DingEmployeeInfo.class)
                .eq(DingEmployeeInfo::getMainJob, "true")
                .eq(StringUtils.isNotBlank(orgCode), DingEmployeeInfo::getOrganizationCode, orgCode)
                .like(StringUtils.isNotBlank(phoneNo), DingEmployeeInfo::getBindUserMobile, phoneNo)
                .like(StringUtils.isNotBlank(name), DingEmployeeInfo::getEmployeeName, name)
                .in(Objects.nonNull(roleCompliantEmployeeCodeList), DingEmployeeInfo::getEmployeeCode, roleCompliantEmployeeCodeList)
                .in(Objects.nonNull(orgNameCompliantOrgCodeList), DingEmployeeInfo::getOrganizationCode, orgNameCompliantOrgCodeList);

        Page<DingEmployeeInfo> page = iDingEmployeeInfoService.page(new Page<>(reqUserInfoListPO.getPageNumber(), reqUserInfoListPO.getPageSize()), wrapper);
        List<DingEmployeeInfo> records = page.getRecords();
        long total = page.getTotal();
        List<ResUserInfoListVO> resUserInfoListVOList = new ArrayList<>();
        if (records != null && records.size() > 0) {
            List<String> employeeCodeList = records.stream().map(DingEmployeeInfo::getEmployeeCode)
                    .distinct().collect(Collectors.toList());

            Map<String, UserInfo> employeeCodeAvailableMap = new HashMap<>();
            if (CollUtil.isNotEmpty(employeeCodeList)) {
                employeeCodeAvailableMap = iUserInfoService.list(Wrappers.lambdaQuery(UserInfo.class)
                        .in(UserInfo::getEmployeeCode, employeeCodeList))
                        .stream().collect(Collectors.toMap(UserInfo::getEmployeeCode, Function.identity()));
            }
            List<String> orgCodeList = records.stream()
                    .filter(r -> StringUtils.isNotBlank(r.getOrganizationCode()))
                    .map(DingEmployeeInfo::getOrganizationCode).distinct()
                    .collect(Collectors.toList());
            Map<String, DingOrganization> orgMap = iDingOrganizationService.getOrgMap(orgCodeList);

            Map<String, UserInfo> finalEmployeeCodeAvailableMap = employeeCodeAvailableMap;
            resUserInfoListVOList = records.stream()
                    .map(r -> {
                        ResUserInfoListVO resListVO = new ResUserInfoListVO();
                        resListVO.setName(r.getEmployeeName());
                        String organizationCode = r.getOrganizationCode();
                        if (StringUtils.isNotBlank(organizationCode) && orgMap.containsKey(organizationCode)) {
                            resListVO.setOrgName(orgMap.get(organizationCode).getOrganizationName());
                            resListVO.setOrgCode(organizationCode);
                            String regionCode = orgMap.get(organizationCode).getDivisionCode();
                            resListVO.setRegionCode(regionCode);
                            if(Objects.nonNull(regionCode)){
                                resListVO.setRegionName(regionCacheHelper.getDisplayName(regionCode,RegionConst.RL_COUNTY));
                            }
                        }
                        // TODO 从用户信息中获取
                        // 从所属组织中获取
//                        resListVO.setRegionId();
                        UserInfo userInfo = finalEmployeeCodeAvailableMap.get(r.getEmployeeCode());
                        // 装配用户状态 及 userId
                        userStatusAssembler(resListVO, userInfo);
                        resListVO.setUpdateTime(r.getUpdateOn());
//                        从用户信息中获取
                        resListVO.setEmployeeCode(r.getEmployeeCode());
                        resListVO.setUserRoleList(getUserRoleVOList(resListVO.getUserId()));
                        return resListVO;
                    }).collect(Collectors.toList());
        }

        pageVo.setTotal(total);
        pageVo.setRecords(resUserInfoListVOList);
        return pageVo;
    }

    private List<String> getOrgNameCompliantOrgCodeList(String orgName) {
        if (StringUtils.isBlank(orgName)) {
            return null;
        }
        List<DingOrganization> dingOrganizationList = iDingOrganizationService.list(Wrappers.lambdaQuery(DingOrganization.class)
                .like(DingOrganization::getOrganizationName, orgName));

        List<String> orgCodeList = dingOrganizationList.stream()
                .map(DingOrganization::getOrganizationCode)
                .collect(Collectors.toList());
        return orgCodeList;
    }

    /**
     * 获取符合查询条件的 用户 employeeCode
     *
     * @param userRoleList
     */
    private List<String> getRoleCompliantEmployeeCodeList(List<UserRoleVO> userRoleList) {
        if (CollUtil.isEmpty(userRoleList)) {
            return null;
        }
        List<Long> roleIdList = iUserRoleService.list()
                .stream().map(UserRole::getRoleId).collect(Collectors.toList());

        List<Long> compliantUserIdList = iUserRoleService.list(Wrappers.lambdaQuery(UserRole.class)
                .in(UserRole::getRoleId, roleIdList)).stream().map(UserRole::getUserId).distinct().collect(Collectors.toList());
        if (CollUtil.isEmpty(compliantUserIdList)) {
            return new ArrayList<>();
        }

        // TODO 这里过滤的非浙政钉用户的专家 后续需要增加补充逻辑
        return iUserInfoService.list(Wrappers.lambdaQuery(UserInfo.class)
                .in(UserInfo::getId, compliantUserIdList))
                .stream().filter(r -> StringUtils.isNotBlank(r.getEmployeeCode()))
                .map(UserInfo::getEmployeeCode).collect(Collectors.toList());

    }

    /**
     * 装配用户状态 及 userId
     *
     * @param resListVO
     * @param userInfo
     */
    private void userStatusAssembler(ResUserInfoListVO resListVO, UserInfo userInfo) {
        if (Objects.nonNull(userInfo)) {
            resListVO.setPhoneNo(userInfo.getMobile());
            if (StringUtils.isNotBlank(userInfo.getAvailable())
                    && UserAvailableEnum.ENABLE.name().equals(userInfo.getAvailable())) {
                resListVO.setStatus(UserAvailableEnum.ENABLE.name());
            } else {
                resListVO.setStatus(UserAvailableEnum.DISABLE.name());
            }
            resListVO.setUserId(userInfo.getId());
        } else {
            resListVO.setStatus(UserAvailableEnum.DISABLE.name());
        }
    }


    private List<UserRoleVO> getUserRoleVOList(Long userId) {
        List<UserRoleVO> userRoleInfoList = new ArrayList<>();
        if (Objects.isNull(userId)) {
            return userRoleInfoList;
        }
        List<UserRole> userRoleList = iUserRoleService.list(Wrappers.lambdaQuery(UserRole.class)
                .eq(UserRole::getUserId, userId));
        if (CollUtil.isNotEmpty(userRoleList)) {
            List<Long> roleIdList = userRoleList.stream().map(UserRole::getRoleId).collect(Collectors.toList());
            List<Role> roleList = iRoleService.list(Wrappers.lambdaQuery(Role.class).in(Role::getId, roleIdList));
            userRoleInfoList = roleList.stream().map(r -> {
                UserRoleVO userRoleVO = new UserRoleVO();
                userRoleVO.setId(r.getId());
                userRoleVO.setName(r.getName());
                return userRoleVO;
            }).collect(Collectors.toList());
        }
        return userRoleInfoList;
    }

    public void disableOrEnable(ReqUserDisableOrEnablePO reqUserDisableOrEnablePO) {
        Long userId = generateUserId(reqUserDisableOrEnablePO.getEmployeeCode());
        UserInfo userInfo = iUserInfoService.getById(userId);
        userInfo.setAvailable(reqUserDisableOrEnablePO.getOperation());
        iUserInfoService.updateById(userInfo);
    }

    public ResUserDetailVO userInfoDetail(ReqUserDetailPO reqUserDetailPO) {
        Long userId = generateUserId(reqUserDetailPO.getEmployeeCode());
        UserInfo userInfo = iUserInfoService.getById(userId);
        if (Objects.isNull(userInfo)) {
            return null;
        }
        ResUserDetailVO resUserDetailVO = new ResUserDetailVO();
        resUserDetailVO.setRealName(userInfo.getRealName());
        resUserDetailVO.setUserId(userInfo.getId());
        resUserDetailVO.setPhoneNo(userInfo.getMobile());
        resUserDetailVO.setStatus(userInfo.getAvailable());

        // 装配用户角色信息列表
        List<UserRole> userRoleList = iUserRoleService.list(Wrappers.lambdaQuery(UserRole.class)
                .eq(UserRole::getUserId, userId));
        List<UserRoleVO> userRoleInfoList = new ArrayList<>();
        if (CollUtil.isNotEmpty(userRoleList)) {
            List<Long> roleIdList = userRoleList.stream().map(UserRole::getRoleId).collect(Collectors.toList());
            List<Role> roleList = iRoleService.list(Wrappers.lambdaQuery(Role.class).in(Role::getId, roleIdList));
            userRoleInfoList = roleList.stream().map(r -> {
                UserRoleVO userRoleVO = new UserRoleVO();
                userRoleVO.setId(r.getId());
                userRoleVO.setName(r.getName());
                return userRoleVO;
            }).collect(Collectors.toList());
        }
        resUserDetailVO.setUserRoleInfoList(userRoleInfoList);
        return resUserDetailVO;
    }

    @Transactional(rollbackFor = Exception.class)
    public void userInfoDetailEdit(ReqUserDetailEditPO reqUserDetailEditPO) {
        String employeeCode = reqUserDetailEditPO.getEmployeeCode();
        Long userId = generateUserId(reqUserDetailEditPO.getEmployeeCode());
        UserInfo userInfo = iUserInfoService.getById(userId);
        if (Objects.isNull(userInfo)) {
            return;
        }
        String phoneNo = reqUserDetailEditPO.getPhoneNo();
        userInfo.setMobile(phoneNo);
        userInfo.setAvailable(reqUserDetailEditPO.getStatus());
        iUserInfoService.updateById(userInfo);

        List<UserRoleVO> userRoleInfoList = reqUserDetailEditPO.getUserRoleInfoList();
        iUserRoleService.remove(Wrappers.lambdaQuery(UserRole.class).eq(UserRole::getUserId, userId));
        if (CollUtil.isNotEmpty(userRoleInfoList)) {
            List<UserRole> userRoleList = userRoleInfoList.stream().map(r -> {
                UserRole saveRecord = new UserRole();
                saveRecord.setRoleId(r.getId());
                saveRecord.setUserId(userId);
                return saveRecord;
            }).collect(Collectors.toList());
            iUserRoleService.saveBatch(userRoleList);
        }

        // 更新浙政钉相关数据
        if (StringUtils.isNotBlank(phoneNo)) {
            iDingEmployeeInfoService
                    .update(Wrappers.lambdaUpdate(DingEmployeeInfo.class)
                            .eq(DingEmployeeInfo::getMainJob, "true")
                            .eq(DingEmployeeInfo::getEmployeeCode, employeeCode)
                            .set(DingEmployeeInfo::getBindUserMobile, phoneNo));
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public Long generateUserId(String employeeCode) {
        UserInfo userInfo = iUserInfoService.getOne(Wrappers.lambdaQuery(UserInfo.class)
                .eq(UserInfo::getEmployeeCode, employeeCode));
        if (Objects.isNull(userInfo)) {
            List<DingEmployeeInfo> dingEmployeeInfoList = iDingEmployeeInfoService
                    .list(Wrappers.lambdaQuery(DingEmployeeInfo.class)
                            .eq(DingEmployeeInfo::getEmployeeCode, employeeCode)
                            .eq(DingEmployeeInfo::getMainJob, "true"));
            if (CollUtil.isEmpty(dingEmployeeInfoList)) {
                throw new BizException("员工账号不存在");
            }
            DingEmployeeInfo dingEmployeeInfo = dingEmployeeInfoList.get(0);
            userInfo = UserInfo.builder()
                    .accountId(dingEmployeeInfo.getAccountId())
                    .username(dingEmployeeInfo.getEmployeeName())
                    .realName(dingEmployeeInfo.getEmployeeName())
                    .employeeCode(dingEmployeeInfo.getEmployeeCode())
                    .available(UserAvailableEnum.DISABLE.name())
                    .createBy(-1L)
                    .updateBy(-1L)
                    .createOn(LocalDateTime.now())
                    .updateOn(LocalDateTime.now())
                    .build();
            iUserInfoService.save(userInfo);
        }
        return userInfo.getId();
    }


    public ResUserDetailVO currentUserInfo() {
        Long userId = LoginUserUtil.getUserId();
        UserInfo userInfo = iUserInfoService.getById(userId);
        if (Objects.isNull(userInfo)) {
            return null;
        }
        ResUserDetailVO resUserDetailVO = new ResUserDetailVO();
        resUserDetailVO.setRealName(userInfo.getRealName());
        resUserDetailVO.setUserId(userInfo.getId());
        resUserDetailVO.setPhoneNo(userInfo.getMobile());
        resUserDetailVO.setStatus(userInfo.getAvailable());
        resUserDetailVO.setEmployeeCode(userInfo.getEmployeeCode());

        // 装配用户角色信息列表
        List<UserRole> userRoleList = iUserRoleService.list(Wrappers.lambdaQuery(UserRole.class)
                .eq(UserRole::getUserId, userId));
        List<UserRoleVO> userRoleInfoList = new ArrayList<>();
        if (CollUtil.isNotEmpty(userRoleList)) {
            List<Long> roleIdList = userRoleList.stream().map(UserRole::getRoleId).collect(Collectors.toList());
            List<Role> roleList = iRoleService.list(Wrappers.lambdaQuery(Role.class).in(Role::getId, roleIdList));
            userRoleInfoList = roleList.stream().map(r -> {
                UserRoleVO userRoleVO = new UserRoleVO();
                userRoleVO.setId(r.getId());
                userRoleVO.setName(r.getName());
                userRoleVO.setCode(r.getCode());
                return userRoleVO;
            }).collect(Collectors.toList());
        }
        resUserDetailVO.setUserRoleInfoList(userRoleInfoList);
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        if (Objects.nonNull(userFullInfo)) {
            resUserDetailVO.setOrgCode(userFullInfo.getOrganizationCode());
            resUserDetailVO.setOrgName(userFullInfo.getOrganizationName());
            resUserDetailVO.setRegionCode(userFullInfo.getRegionCode());
        }
        return resUserDetailVO;
    }

}
